 /**    
  *******************************************************************************************************   
  * @attention    
  * Copyright (c) 2023, SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICROELECTRONICS./ FUDAN MICRO.)    
  * All rights reserved.    
  *    
  * Processor:                   FM33FG0xxA    
  * http:                        http://www.fmdevelopers.com.cn/    
  *    
  * Redistribution and use in source and binary forms, with or without    
  * modification, are permitted provided that the following conditions are met    
  *    
  * 1. Redistributions of source code must retain the above copyright notice,    
  *    this list of conditions and the following disclaimer.    
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,    
  *    this list of conditions and the following disclaimer in the documentation    
  *    and/or other materials provided with the distribution.    
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors    
  *    may be used to endorse or promote products derived from this software    
  *    without specific prior written permission.    
  *    
  * 4. To provide the most up-to-date information, the revision of our documents     
  *    on the World Wide Web will be the most Current. Your printed copy may be      
  *    an earlier revision. To verify you have the latest information avaliable,    
  *    refer to: http://www.fmdevelopers.com.cn/.    
  *    
  * THIS SOFTWARE IS PROVIDED BY FUDAN MICRO "AS IS" AND ANY EXPRESSED:     
  * ORIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES     
  * OF MERCHANTABILITY NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE    
  * ARE DISCLAIMED.IN NO EVENT SHALL FUDAN MICRO OR ITS CONTRIBUTORS BE LIABLE     
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS     
  * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER    
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,     
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISINGIN ANY WAY OUT OF THE     
  * USE OF THIS SOFTWARE, EVEN IF ADVISED OFTHE POSSIBILITY OF SUCH DAMAGE.  
  *
  ******************************************************************************************************* 
  */     
#include "main.h"
#include "fm33fg0xxa_fl.h"
#include "iwdt.h"
#include "svd.h"
#include "rmu.h"
#include "led.h"
#include "uart.h"
#include "lin.h" 
#include "can.h"
#include "canfd.h"
#include "pll.h"
#include "bootloader.h"
#include "lptim16.h"

/**
  * @brief  ECC检错中断不可屏蔽，直接进入NMI中断服务 请保留 
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
    uint32_t RAMDetectionFlag = 0, DataFlashDetectionFlag = 0, CodeFlashDetectionFlag = 0;
    uint32_t WakeUpDebugFlag = 0;
    
    /* RAM ECC 检错 必须保留 */
    RAMDetectionFlag = FL_FLASH_IsActiveFlag_ECCDetectionInRAM(FLASH);
    if (0x01U == RAMDetectionFlag)
    {   
        FL_FLASH_ClearFlag_ECCDetectionInRAM(FLASH);
        /* 客户安全处理 */
        /* ...... */
    }
    
    /* Data Flash ECC检错 必须保留 */ 
    DataFlashDetectionFlag = FL_FLASH_IsActiveFlag_ECCDetectionInDataFlash(FLASH);
    if (0x01U == DataFlashDetectionFlag)
    {
        FL_FLASH_ClearFlag_ECCDetectionInDataFlash(FLASH); 
        /* 客户安全处理 */
        /* ...... */
    }
    
    /* Code Flash ECC检错 必须保留 */ 
    CodeFlashDetectionFlag = FL_FLASH_IsActiveFlag_ECCDetectionInCodeFlash(FLASH);
    if (0x01U == CodeFlashDetectionFlag)
    {
        FL_FLASH_ClearFlag_ECCDetectionInCodeFlash(FLASH); 
        /* 客户安全处理 */
        /* ...... */
    }
    
    /* CPU Debugger唤醒 必须保留 */
    WakeUpDebugFlag = FL_PMU_IsActiveFlag_WakeupDBG(PMU);
    if (0x01U == WakeUpDebugFlag)
    {
        FL_PMU_ClearFlag_WakeupDBG(PMU);
    }
}

/**
  * @brief  HardFault 中断服务函数 请保留 
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{   /* 软件复位MCU，使用内联函数 */
    RMU_Soft_SystemReset();
}

/**
  * @brief  掉电监控，用于执行掉电事件
  * @param  None
  * @retval None
  */
static void PowerDownMonitoring(void)
{
    /* 确认SVD监测结果是否低于阈值 */
    if (true == SVD_Result_Confirmed(SVD_BELOW_THRESHOLD, 1U))
    {   /* 防抖处理 */
        if (true == SVD_Result_Confirmed(SVD_BELOW_THRESHOLD, 2000U/*us*/))
        {
            /* 电压下降到报警阈值，处理掉电事件 */
            /* 用户程序1 */
            /* ...... */
            /* 注意：用户需确保在BOR复位之前执行完毕 */
            
            /* 确认SVD监测结果是否高于阈值，如否则持续等待 */
            while(false == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 20U/*us*/));
            /* 由于电源有下降到报警阈值，软件复位后重新初始化 */
            RMU_Soft_SystemReset();
        }
    }
}


static int32_t(*const TxMsgFill[BOOT_COMM_CHL])(uint8_t *, uint32_t) = 
{
    LIN_TxMsgFill, /* BOOT向LIN填充发送数据，LIN定义为通道0 */
    CAN_TxMsgFill,  /* BOOT向CAN填充发送数据，CAN定义为通道1 */
    CANFD_TxMsgFill  /* BOOT向CANFD填充发送数据，CANFD定义为通道2 */
};


int main(void)
{   
    /* 立即设置向量表 */
    SCB->VTOR = APP_START_ADDR;
    
    /* 使能IWDT */
    (void)IWDT_Init(FL_IWDT_PERIOD_4000MS);
    
    /* 延时函数初始化 */
    FL_Init();
    
    /* 使能SVD,阈值4.16V(falling)~4.284V(rising) */
    (void)SVD_Init(SVD_MONTIOR_VDD,FL_SVD_WARNING_THRESHOLD_GROUP11,FL_SVD_REFERENCE_1P0V);
    
    /* 确认SVD监测结果是否高于阈值，如否则持续等待 */
    while(false == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 2000U/*us*/));
    
    /* 使能BOR为2.0V */
    RMU_BOR_Init(FL_RMU_BOR_THRESHOLD_2P00V);
    
    /* 检查APP执行标志是否置起，如未置起则写入标志并复位 */
    BOOT_Object.AppExeFlagCheck();
    
    /* 有APP，执行BOOT参数配置 */
    BOOT_Object.Configuration();
    
    /* BOOT屏蔽总中断，APP此处需开启 */
    __set_PRIMASK(0);
    
    /* 先将外部XTHF分频到1M，再利用PLL锁相环倍频至32M，具体请见初始化参数 */
    (void)SelXTHFToPLL(&CMU_XTHF_Init_Data, &CMU_PLL_Init_Data);
    
    /* 用于监控XTHF停振事件和PLL失锁事件，先配置系统时钟，再使能该监控定时器, 10ms周期 */
    LPTIM16_Init();
    
    /* Initialize all configured peripherals */
    /* SHOULD BE KEPT!!! */
    MF_Config_Init();  /* 当使用到SVD作为唤醒源时，PMU_CR只允许配置Normal Sleep模式 */
    
    /* LIN初始化 */
    lin_init();
    
    /* CAN初始化 */
    (void)Can_Controller_Init();
    
    /* CANFD外设初始化 */
    (void)CanFD_Init(CANFD1);
    
    LED_ON(RED);   /* 调试信息灯 */
    FL_DelayMs(50);
    LED_OFF(RED);

    while(1)
    {
        static uint32_t Time_10ms_Cnt = 0;
        
        /* 清狗 */
        IWDT_Clr();
        
        /* 电源掉电监测处理 */
        PowerDownMonitoring();
        
        /* LIN接收帧处理 */
        if (0x1 == scheduleTable[0].FullFlg)
        {
            uint32_t primask;
            /* 中断全局变量，竞态保护 */
            primask = __get_PRIMASK();
            __disable_irq();
            scheduleTable[0].FullFlg = 0;
            __set_PRIMASK(primask);
            /* 填充到BOOT Buffer */
            BOOT_Object.RxMsgFill(scheduleTable[0].data, scheduleTable[0].length, 0/*LIN定义为通道0*/);
        }
        
        /* CAN接收帧处理 */
        if (0x1 == RxMessage.FullFlg)
        {         
            uint32_t primask;
            /* 中断全局变量，竞态保护 */
            primask = __get_PRIMASK();
            __disable_irq();
            RxMessage.FullFlg = 0;
            __set_PRIMASK(primask);
            /* 填充到BOOT Buffer */
            BOOT_Object.RxMsgFill(RxMessage.Data, RxMessage.DLC, 1/*CAN定义为通道1*/);
        }
        
        /* CANFD接收帧处理 */
        if (0x1 == CANFD_RxMessage.FullFlg)
        {         
            uint32_t primask;
            /* 中断全局变量，竞态保护 */
            primask = __get_PRIMASK();
            __disable_irq();
            CANFD_RxMessage.FullFlg = 0;
            __set_PRIMASK(primask);
            /* 填充到BOOT Buffer */
            BOOT_Object.RxMsgFill(CANFD_RxMessage.Data.Data8, (CANFD_RxMessage.PTR >> 28), 2/*CANFD定义为通道2*/);
        }        
        
        /* 功能执行 */
        BOOT_Object.Process(TxMsgFill);
        
        if (1 == Period10msFlag) 
        {
            Period10msFlag = 0;
            /* 判断是否发生XTHF停振或PLL失锁事件，如有及时切回主系统时钟 */
            SwtichToMainSystemClock(&CMU_RCHF_Init_Data, &CMU_XTHF_Init_Data, &CMU_PLL_Init_Data);
            
            /* 运行指示灯 */
            if (++Time_10ms_Cnt > 10)
            {
                LED_TOG(BLUE);
                Time_10ms_Cnt = 0;
            }
        }
    }
}


