/**
  ******************************************************************************
  * @attention
  * Copyright (c) 2022, SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICROELECTRONICS./ FUDAN MICRO.)
  * All rights reserved.
  *
  * Processor:                   FM33FG0xxA
  * http:                        http://www.fmdevelopers.com.cn/
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met
  *
  * 1. Redistributions of source code must retain the above copyright notice,
  *    this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *
  * 3. Neither the name of the copyright holder nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
  * 4. To provide the most up-to-date information, the revision of our documents
  *    on the World Wide Web will be the most Current. Your printed copy may be
  *    an earlier revision. To verify you have the latest information avaliable,
  *    refer to: http://www.fmdevelopers.com.cn/.    
  *
  * THIS SOFTWARE IS PROVIDED BY FUDAN MICRO "AS IS" AND ANY EXPRESSED
    ORIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
    OF MERCHANTABILITY NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.IN NO EVENT SHALL FUDAN MICRO OR ITS CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISINGIN ANY WAY OUT OF THE
    USE OF THIS SOFTWARE, EVEN IF ADVISED OFTHE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

#include "pll.h"

#define PLL_TIMEOUT            0x00000FFFU  /* 约90ms */ 
#define XTHF_TIMEOUT           0x0003FFFFU  /* 约10s */

/* XTHF寄存器参数检查 */
#define XTHF_REG_CHECK(xthf)  ((FL_CMU_XTHF_GetStartWaitTime()   == (xthf)->StartWaitTime) &&\
                               (FL_CMU_XTHF_IsEnabledBypass()    == (xthf)->Bypass) &&\
                               (FL_CMU_XTHF_ReadDriverStrength() == (xthf)->DriverStrength) &&\
                               (FL_CMU_XTHF_IsEnabled()          == FL_ENABLE))

/* 系统时钟选择错误变量 */
volatile uint32_t SYSCLKSELErrorState = 0;
/* 系统时钟状态变量 */
volatile SYSCLK_STATE SYSCLKState = SYSCLK_NORMAL;

/* RCHF初始化参数 */
const FL_CMU_RCHF_InitTypeDef CMU_RCHF_Init_Data = {
    /* RCHF频率必须≤24MHz, 否则需要考虑Code Wait */
    /* 当系统时钟使用PLL时，RCHF频率和XTHF频率可以不一致，因为可以通过PLL得到相同目标频率，推荐固定为8MHz */
    FL_CMU_RCHF_FREQUENCY_8MHZ
};

/* XTHF初始化参数 */
const FL_CMU_XTHF_InitTypeDef CMU_XTHF_Init_Data = {
    /* XTHF频率值，必须与外部实际晶体匹配 */
    8000000,
    /* XTHF启动等待时间 */
    FL_CMU_XTHF_START_WAIT_CYCLE_16384,
    /* XTHF Bypss使能或禁止 */
    FL_DISABLE,
    /* XTHF振荡强度 */
    0xA,
};

/* PLL初始化参数 */
const FL_CMU_PLL_InitTypeDef CMU_PLL_Init_Data = {
    /* PLL将1M时钟倍频至32MHz */
    32,  
    /* PLL将参考时钟预分频至1MHz*/
    FL_CMU_PLL_PSC_DIV8,
    /* PLL时钟源输入选择 */
    FL_CMU_PLL_CLK_SOURCE_XTHF,
};


/**
  * @brief  ECCC_RAMP_FDET_IRQHandler中断服务
  * @param  void
  * @retval None
  */
void ECCC_RAMP_FDET_IRQHandler(void)
{
    uint32_t SYSCLKSELErrorIT = 0;
    uint32_t SYSCLKSELErrorFlag = 0;
    /* Note: CHIP U01版芯片在使能PLL后必须禁止ECCC_RAMP_FDET_IRQn中断服务，所以不应触发 */
    assert_param((!(CHIP_U01)) || (0x0 == FL_CMU_PLL_IsEnabled()));
    
    /* 发生XTHF停振或PLL失锁事件，切到次系统时钟 */
    if (((0x1U == FL_CMU_IsEnabledIT_XTHFFail()) && (0x1U == FL_CMU_IsActiveFlag_XTHFFail())) ||\
        ((0x1U == FL_CMU_IsEnabledIT_PLLLostLock()) && (0x1U == FL_CMU_IsActiveFlag_PLLLostLock())) )
    {   /* 清XTHF停振标识和PLL失锁标识在下面函数内执行 */
        SwtichToSenondarySystemClock(&CMU_RCHF_Init_Data, &CMU_PLL_Init_Data);
    }
    
    /* 时钟选择中断处理 */
    SYSCLKSELErrorIT = FL_CMU_IsEnabledIT_SYSCKEWrong();
    SYSCLKSELErrorFlag = FL_CMU_IsActiveFlag_SYSCSEWrong();
    if ((0x01U == SYSCLKSELErrorIT) && (0x01U == SYSCLKSELErrorFlag))
    {   /* 清除系统时钟选择错误中断标志 */
        FL_CMU_ClearFlag_SYSCSEWrong();
        SYSCLKSELErrorState = 1;
    }
}


/**
  * @brief  RCHF初始化配置
  * @param  RCHFclock  RCHF参考时钟选择 This parameter can be one of the following values:
  *           @arg @ref FL_CMU_RCHF_FREQUENCY_8MHZ
  *           @arg @ref FL_CMU_RCHF_FREQUENCY_16MHZ
  *           @arg @ref FL_CMU_RCHF_FREQUENCY_24MHZ
  *           @arg @ref FL_CMU_RCHF_FREQUENCY_32MHZ
  * @retval PLL分频到1M的参量
  */
static uint32_t RCHFInit(uint32_t RCHFclock)
{   
    /* PLL分频到1M的参量 */
    uint32_t PLL_Prescaler_1M;
    
    switch (RCHFclock)
    {
        case FL_CMU_RCHF_FREQUENCY_8MHZ:
            FL_CMU_RCHF_WriteTrimValue(RCHF8M_TRIM);
            PLL_Prescaler_1M = FL_CMU_PLL_PSC_DIV8;
            break;

        case FL_CMU_RCHF_FREQUENCY_16MHZ:
            FL_CMU_RCHF_WriteTrimValue(RCHF16M_TRIM);
            PLL_Prescaler_1M = FL_CMU_PLL_PSC_DIV16;
            break;

        case FL_CMU_RCHF_FREQUENCY_24MHZ:
            FL_CMU_RCHF_WriteTrimValue(RCHF24M_TRIM);
            PLL_Prescaler_1M = FL_CMU_PLL_PSC_DIV24;
            break;

        case FL_CMU_RCHF_FREQUENCY_32MHZ:
            FL_CMU_RCHF_WriteTrimValue(RCHF32M_TRIM);
            PLL_Prescaler_1M = FL_CMU_PLL_PSC_DIV32;
            break;

        default:
            FL_CMU_RCHF_WriteTrimValue(RCHF8M_TRIM);
            PLL_Prescaler_1M = FL_CMU_PLL_PSC_DIV8;
            break;
    }

    FL_CMU_RCHF_SetFrequency(RCHFclock);
    FL_CMU_RCHF_Enable();
    return PLL_Prescaler_1M;
}


/**
  * @brief  切换到次系统时钟
  * @param  const FL_CMU_RCHF_InitTypeDef *rchf
  *             RCHF参数
  *         const FL_CMU_PLL_InitTypeDef *pll
  *             PLL参数
  * @retval None
  */
void SwtichToSenondarySystemClock(const FL_CMU_RCHF_InitTypeDef *rchf,
                                  const FL_CMU_PLL_InitTypeDef  *pll)
{
    volatile uint32_t counter; 
    uint32_t PLL_Prescaler_1M;
    /* 通过内核时钟计算100us延时的计数个数 */
    uint32_t RCLP_Clock = (SystemCoreClock / 1000000) * 100;
    
    PLL_Prescaler_1M = RCHFInit(rchf->Frequency);
    FL_CMU_SetSystemClockSource(FL_CMU_SYSTEM_CLK_SOURCE_RCHF);
    FL_CMU_PLL_Disable();
    RCC_PLL_ConfigDomain_SYS(FL_CMU_PLL_CLK_SOURCE_RCHF, PLL_Prescaler_1M, pll->Multiplier);
    FL_CMU_PLL_Enable();
    counter = PLL_TIMEOUT;
    /* 等待PLL建立，3个RCLP Clock，约91.55us */
    while((0x0 == FL_CMU_IsActiveFlag_PLLReady()) && (--counter));
    /* 必须等待3个RCLP的Clock，确保初始化PLL过程中如产生PLL失锁标志时可得以清除 */
    for (counter = 0; counter < RCLP_Clock; counter++)
    {
        __NOP();
    }
    /* 清XTHF停振中断标识 */
    FL_CMU_ClearFlag_XTHFFail();
    /* 清PLL失锁中断标识 */
    FL_CMU_ClearFlag_PLLLostLock();
    /* 禁止高频检测报警中断 */
    FL_CMU_DisableIT_XTHFFail();
    /* 禁止PLL失锁报警中断 */
    FL_CMU_DisableIT_PLLLostLock();
    
    FL_CMU_SetSystemClockSource(FL_CMU_SYSTEM_CLK_SOURCE_PLL);
    SYSCLKState = SYSCLK_ERROR;
    SystemCoreClockUpdate();
}


/**
  * @brief  切换到主系统时钟
  * @param  const FL_CMU_RCHF_InitTypeDef *rchf,
  *              RCHF参数
  *         const FL_CMU_XTHF_InitTypeDef *rchf
  *              XTHF参数
  *         const FL_CMU_PLL_InitTypeDef *pll
  *              PLL参数
  * @retval None
  */
void SwtichToMainSystemClock(const FL_CMU_RCHF_InitTypeDef *rchf,
                             const FL_CMU_XTHF_InitTypeDef *xthf,
                             const FL_CMU_PLL_InitTypeDef  *pll)
{
    if ((SYSCLK_ERROR == SYSCLKState) &&\
        ((FL_CMU_SYSTEM_CLK_SOURCE_PLL != FL_CMU_GetSystemClockSource()) ||\
         (pll->ClockSource != FL_CMU_PLL_GetClockSource())))
    { 
        /* 即使发生停振或失锁事件，XTHF寄存器参数不应发生变化 */
        if (XTHF_REG_CHECK(xthf) || XTHF_REG_CHECK(xthf))
        {
            volatile uint32_t Reday[3];  /* 避免编译器优化 */
            uint32_t counter, primask;
            
            /* 判断XTHF稳定标志是否建立，连续读三次 */
            Reday[0] = FL_CMU_XTHF_IsReady();
            Reday[1] = FL_CMU_XTHF_IsReady();
            Reday[2] = FL_CMU_XTHF_IsReady();
            if (Reday[0] && Reday[1] && Reday[2])
            {
                /* 清XTHF停振中断标识 */
                FL_CMU_ClearFlag_XTHFFail(); 
                if (0x0 == FL_CMU_IsActiveFlag_XTHFFail())
                {
                    /* 准备切换到主系统时钟相关工作 */
                    (void)RCHFInit(rchf->Frequency);
                    /* 竞态保护 */
                    primask = __get_PRIMASK();
                    __disable_irq();
                    FL_CMU_SetSystemClockSource(FL_CMU_SYSTEM_CLK_SOURCE_RCHF);
                    FL_CMU_PLL_Disable();
                    RCC_PLL_ConfigDomain_SYS(pll->ClockSource, pll->Prescaler, pll->Multiplier);
                    FL_CMU_PLL_Enable();
                    /* 等待PLL建立，3个RCLP Clock，约91.55us */
                    counter = PLL_TIMEOUT;
                    do
                    {   /* 判断PLL锁住标志，连续读三次 */
                        Reday[0] = FL_CMU_IsActiveFlag_PLLReady();
                        Reday[1] = FL_CMU_IsActiveFlag_PLLReady();
                        Reday[2] = FL_CMU_IsActiveFlag_PLLReady();
                        if (Reday[0] && Reday[1] && Reday[2])
                        {
                            volatile uint32_t counter = 0;
                            /* 通过内核时钟计算100us延时的计数个数 */
                            uint32_t RCLP_Clock = (SystemCoreClock / 1000000) * 100;
                            /* 必须等待3个RCLP的Clock，确保初始化PLL过程中如产生PLL失锁标志时可得以清除 */
                            for (counter = 0; counter < RCLP_Clock; counter++)
                            {
                                __NOP();
                            }
                            /* 清PLL失锁中断标识 */
                            FL_CMU_ClearFlag_PLLLostLock();
                            if (FL_CMU_IsActiveFlag_XTHFFail()) break; 
                            if (!(CHIP_U01))
                            {   /* Note: CHIP U01不能判断失锁标志 */
                                if (FL_CMU_IsActiveFlag_PLLLostLock()) break;
                                /* Note: CHIP U01在使能PLL后必须禁止ECCC_RAMP_FDET_IRQn中断服务 */
                                SYSCLKState = SYSCLK_NORMAL;
                                FL_CMU_EnableIT_XTHFFail();
                                FL_CMU_EnableIT_PLLLostLock();
                            }
                            else
                            {
                                SYSCLKState = SYSCLK_NORMAL;
                            }
                            FL_CMU_SetSystemClockSource(FL_CMU_SYSTEM_CLK_SOURCE_PLL);
                            SystemCoreClockUpdate();
                            break;
                        }
                    }while((0x0 == FL_CMU_IsActiveFlag_XTHFFail()) && (--counter));
                    /* 未成功切换到主系统时钟，则恢复到次系统时钟 */
                    if (SYSCLK_NORMAL != SYSCLKState)
                    {
                        SwtichToSenondarySystemClock(rchf, pll);
                    }
                    /* 取消竞态保护 */
                    __set_PRIMASK(primask);
                }
            }
        }
        else
        {   
            (void)RCHFInit(rchf->Frequency);
            FL_CMU_SetSystemClockSource(FL_CMU_SYSTEM_CLK_SOURCE_RCHF);
            FL_CMU_PLL_Disable();
            /* 禁止高频检测报警中断 */
            FL_CMU_DisableIT_XTHFFail();
            /* 禁止PLL失锁报警中断 */
            FL_CMU_DisableIT_PLLLostLock();
            SYSCLKState = SYSCLK_NORMAL;
            (void)SelXTHFToPLL(&CMU_XTHF_Init_Data, &CMU_PLL_Init_Data);
        }
    }
}


/**
  * @brief
  * @rmtoll   PLLCR    OSEL    LL_RCC_GetPLLDigitalDomainOutput
  * @param    Source This parameter can be one of the following values:
  *           @arg @ref FL_CMU_PLL_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_PLL_CLK_SOURCE_XTHF
  * @param    PLL_R 锁相环的参考时钟需要预分频至1M，再进行PLL倍频 This parameter can be one of the following values:
  *           @arg @ref FL_CMU_PLL_PSC_DIV1
  *           @arg @ref FL_CMU_PLL_PSC_DIV2
  *           @arg @ref FL_CMU_PLL_PSC_DIV4
  *           @arg @ref FL_CMU_PLL_PSC_DIV8
  *           @arg @ref FL_CMU_PLL_PSC_DIV12
  *           @arg @ref FL_CMU_PLL_PSC_DIV16
  *           @arg @ref FL_CMU_PLL_PSC_DIV24
  *           @arg @ref FL_CMU_PLL_PSC_DIV32
  * @param    PLL_DB This parameter can be Between parameters of the following values:
  *           @arg @ref

  * @retval   None
  */
void RCC_PLL_ConfigDomain_SYS(uint32_t Source, uint32_t PLL_R, uint32_t PLL_DB)
{
    MODIFY_REG(CMU->PLLCR, CMU_PLLCR_DB_Msk | CMU_PLLCR_REFPRSC_Msk | CMU_PLLCR_INSEL_Msk,
               ((PLL_DB - 1) << CMU_PLLCR_DB_Pos) | PLL_R | Source);
}


/**
  * @brief  XTHF初始化配置
  * @param  const FL_CMU_XTHF_InitTypeDef *init
               XTHF初始化参数
  * @retval None
  */
void XTHFInit(const FL_CMU_XTHF_InitTypeDef *init)
{
    uint32_t counter = 0;
    uint32_t readystatus = 0;
    
    FL_GPIO_InitTypeDef    gpioInitStruction = { 0x0 };

    /* PC2, PC3配置成模拟功能，外接XTHF */
    gpioInitStruction.pin           = FL_GPIO_PIN_2 | FL_GPIO_PIN_3;
    gpioInitStruction.mode          = FL_GPIO_MODE_ANALOG;
    gpioInitStruction.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    gpioInitStruction.pull          = FL_GPIO_BOTH_DISABLE;
    gpioInitStruction.remapPin      = FL_GPIO_PINREMAP_FUNCTON0;
    gpioInitStruction.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    (void)FL_GPIO_Init(GPIOC, &gpioInitStruction);
    /* 发生晶体停振事件，禁止复位芯片，仅产生报警 */
    FL_RMU_DisableCrystalStopReset(RMU);
    /* 配置XTHF参数 */
    FL_CMU_XTHF_WriteDriverStrength(init->DriverStrength);
    (void)((FL_ENABLE == init->Bypass) ? FL_CMU_XTHF_EnableBypass() : FL_CMU_XTHF_DisableBypass());
    FL_CMU_XTHF_SetStartWaitTime(init->StartWaitTime);
    /* 使能XTHF */
    FL_CMU_XTHF_Enable();
    do
    {
        readystatus = FL_CMU_XTHF_IsReady();
        counter++;
    
        if ((counter % 100) == 0)
        {   /* 清狗 */
            FL_IWDT_ReloadCounter(IWDT);
        }
    } while((readystatus != 0x1U) && (counter != XTHF_TIMEOUT));  /* 等待XTHF建立 */
    /* 清除高频晶体停振检测标志，必须放在XTHF稳定建立之后 */
    FL_CMU_ClearFlag_XTHFFail();
    
    /* 写晶体频率值，获取系统时钟使用 */
    XTHFClock = init->Frequency;
}


/**
  * @brief  选择外部XTHF作为锁相环的参考时钟
  * @param    const FL_CMU_XTHF_InitTypeDef *xthfInit
  *              XTHF初始化参数
  *           const FL_CMU_PLL_InitTypeDef *pllInit
  *              PLL初始化参数
  * @note   PLL输出时钟频率为1M*（pllInit->Multiplier），输出频率最高不大于64M，最低不小于32MHz
  * @retval None
  */
uint32_t SelXTHFToPLL(const FL_CMU_XTHF_InitTypeDef *xthfInit, 
                      const FL_CMU_PLL_InitTypeDef   *pllInit)
{
    volatile uint32_t counter = 0;
    uint32_t readystatus = 0;
    uint32_t DataWait;
    uint32_t PLLclock;
    /* 通过内核时钟计算100us延时的计数个数 */
    uint32_t RCLP_Clock = (SystemCoreClock / 1000000) * 100;

    if ((pllInit->Multiplier > 64U) || (pllInit->Multiplier < 32U) ||\
        (pllInit->ClockSource != FL_CMU_PLL_CLK_SOURCE_XTHF))
    {   /* 超出PLL倍频最高范围或PLL时钟源非XTHF */
        return 1;
    }
    else
    {  
        /* 先配置Wait，降低访问Flash总线时钟 */ 
        FL_FLASH_SetCodeReadWait(FLASH, FL_FLASH_READ_CODEWAIT_2CYCLE);
        FL_FLASH_SetDataReadWait(FLASH, FL_FLASH_READ_DATAWAIT_5CYCLE);
        /* XTHF配置 */
        XTHFInit(xthfInit);
        /* 发生PLL失锁事件，禁止复位芯片，仅产生报警 */
        FL_RMU_DisablePLLLostLockReset(RMU);
        /* 配置PLL寄存器 */
        RCC_PLL_ConfigDomain_SYS(FL_CMU_PLL_CLK_SOURCE_XTHF, pllInit->Prescaler, pllInit->Multiplier);
        /* 使能PLL */
        FL_CMU_PLL_Enable();
        counter = 0x0;
        do
        {
            readystatus = FL_CMU_IsActiveFlag_PLLReady();
            counter++;
        
            if ((counter % 100) == 0)
            {   /* 清狗 */
                FL_IWDT_ReloadCounter(IWDT);  
            }
        } while((readystatus != 0x1U) && (counter != PLL_TIMEOUT));  /* 等待PLL建立 */
        /* 必须等待3个RCLP的Clock，确保初始化PLL过程中如产生PLL失锁标志时可得以清除 */
        for (counter = 0; counter < RCLP_Clock; counter++)
        {
            __NOP();
        }
        /* 清除PLL失锁中断标志，必须放置在PLL成功锁定之后 */
        FL_CMU_ClearFlag_PLLLostLock();
        /* 清除时钟选择错误标志 */
        FL_CMU_ClearFlag_SYSCSEWrong();
        
        /* 注意：系统时钟为AHB分频后时钟，此处不分频 */
        FL_CMU_SetAHBPrescaler(FL_CMU_AHBCLK_PSC_DIV1);
        FL_CMU_SetAPB1Prescaler(FL_CMU_APB1CLK_PSC_DIV1);
        FL_CMU_SetAPB2Prescaler(FL_CMU_APB2CLK_PSC_DIV1);
        FL_CMU_SetSystemClockSource(FL_CMU_SYSTEM_CLK_SOURCE_PLL);
        
        PLLclock = pllInit->Multiplier;
        switch (FL_CMU_GetAHBPrescaler())
        {
            case FL_CMU_AHBCLK_PSC_DIV2:
                PLLclock >>= 1;
                break;
            case FL_CMU_AHBCLK_PSC_DIV4:
                PLLclock >>= 2;
                break;
            case FL_CMU_AHBCLK_PSC_DIV8:
                PLLclock >>= 3;
                break;
            case FL_CMU_AHBCLK_PSC_DIV16:
                PLLclock >>= 4;
                break;
            default:
                break;
        }
        
        /* CODE Flash读等待周期配置，CODE Flash总线时钟频率需小于等于24MHz */ 
        if (PLLclock <= 24) 
        {   
            FL_FLASH_SetCodeReadWait(FLASH, FL_FLASH_READ_CODEWAIT_0CYCLE);
        }
        else if (PLLclock <= 48) /* (clock > 24) && (clock <= 48) */
        {   
            FL_FLASH_SetCodeReadWait(FLASH, FL_FLASH_READ_CODEWAIT_1CYCLE);
        }
        else  /* (clock > 48) && (clock <= 64) */
        {
            FL_FLASH_SetCodeReadWait(FLASH, FL_FLASH_READ_CODEWAIT_2CYCLE);
        }
        
        /* DATA Flash读等待周期配置，DATA Flash总线时钟频率需小于等于12.5MHz */
        PLLclock *= 10;
        DataWait = (PLLclock / 125);
        if (DataWait && (0 == (PLLclock % 125)))
        {
            DataWait -= 1;
        }
        DataWait = FL_FLASH_READ_DATAWAIT_0CYCLE + FL_FLASH_READ_DATAWAIT_1CYCLE * DataWait;
        FL_FLASH_SetDataReadWait(FLASH, DataWait);
    }
    
    if (CHIP_U01)
    {   /* Note: CHIP U01版芯片在使能PLL后必须禁止ECCC_RAMP_FDET_IRQn中断服务 */
        /* 禁止高频检测报警中断 */
        FL_CMU_DisableIT_XTHFFail();
        /* 禁止PLL失锁中断 */
        FL_CMU_DisableIT_PLLLostLock();
        /* 禁止时钟选择错误中断 */
        FL_CMU_DisableIT_SYSCKEWrong();
        /* 禁止ECCC_RAMP_FDET_IRQn中断服务 */
        NVIC_DisableIRQ(ECCC_RAMP_FDET_IRQn);
        NVIC_ClearPendingIRQ(ECCC_RAMP_FDET_IRQn);
    }
    else
    {   /* 监控PLL失锁事件功能配置 */
        FL_NVIC_ConfigTypeDef InterruptConfigStruct;
        /* 清除高频晶体停振检测标志 */
        FL_CMU_ClearFlag_XTHFFail();
        /* 使能高频检测报警中断 */
        FL_CMU_EnableIT_XTHFFail();
        /* 清除PLL失锁中断标志 */
        FL_CMU_ClearFlag_PLLLostLock();        
        /* 使能PLL失锁中断 */
        FL_CMU_EnableIT_PLLLostLock();
        /* 清除时钟选择错误标志 */
        FL_CMU_ClearFlag_SYSCSEWrong();
        /* 使能时钟选择错误中断 */
        FL_CMU_EnableIT_SYSCKEWrong();
        /* 使能ECCC_RAMP_FDET_IRQn中断服务 */
        InterruptConfigStruct.preemptPriority = 0x00;
        FL_NVIC_Init(&InterruptConfigStruct, ECCC_RAMP_FDET_IRQn);
    }
    SystemCoreClockUpdate();
    return 0;
}

